@@ -2,11 +2,22 @@ use std::{mem::MaybeUninit, borrow::Cow, ffi::c_void};
 
 use crate::lua::*;
 
+use crate::userdata::TaggedUserData;
+
 #[repr(transparent)]
 #[derive(Clone, Copy, Debug)]
 pub struct LuaState(pub *mut std::ffi::c_void);
 unsafe impl Send for LuaState {}
 impl LuaState {
+	pub unsafe fn new() -> Result<Self, LuaError> {
+		let lua = (LUA_SHARED.lual_newstate)();
+		(LUA_SHARED.lual_openlibs)(lua);
+		if lua.is_null() {
+			Err(LuaError::MemoryAllocationError)
+		} else {
+			Ok(lua)
+		}
+	}
 
 	/// Returns the Lua string as a slice of bytes.
 	///
@@ -109,6 +120,11 @@ impl LuaState {
 		(LUA_SHARED.lua_pushvalue)(*self, index)
 	}
 
+	#[inline]
+	pub unsafe fn push_lightuserdata(&self, data: *mut c_void) {
+		(LUA_SHARED.lua_pushlightuserdata)(*self, data)
+	}
+
 	#[inline]
 	pub unsafe fn get_field(&self, index: i32, k: LuaString) {
 		(LUA_SHARED.lua_getfield)(*self, index, k)
@@ -148,6 +164,15 @@ impl LuaState {
 		}
 	}
 
+	pub unsafe fn load_buffer(&self, buff: &[u8], name: LuaString) -> Result<(), LuaError> {
+		let lua_error_code = (LUA_SHARED.lual_loadbuffer)(*self, buff.as_ptr() as LuaString, buff.len(), name);
+		if lua_error_code == 0 {
+			Ok(())
+		} else {
+			Err(LuaError::from_lua_state(*self, lua_error_code))
+		}
+	}
+
 	pub unsafe fn load_file(&self, path: LuaString) -> Result<(), LuaError> {
 		let lua_error_code = (LUA_SHARED.lual_loadfile)(*self, path);
 		if lua_error_code == 0 {
@@ -272,8 +297,8 @@ impl LuaState {
 	}
 
 	#[inline]
-	pub unsafe fn check_userdata(&self, arg: i32, name: LuaString) -> *mut c_void {
-		(LUA_SHARED.lual_checkudata)(*self, arg, name)
+	pub unsafe fn check_userdata(&self, arg: i32, name: LuaString) -> *mut TaggedUserData {
+		(LUA_SHARED.lual_checkudata)(*self, arg, name) as *mut _
 	}
 
 	pub unsafe fn test_userdata(&self, index: i32, name: LuaString) -> bool {
@@ -310,6 +335,12 @@ impl LuaState {
 		(LUA_SHARED.lual_checknumber)(*self, arg)
 	}
 
+	#[inline]
+	pub unsafe fn check_boolean(&self, arg: i32) -> bool {
+		(LUA_SHARED.lual_checktype)(*self, arg, LUA_TBOOLEAN);
+		(LUA_SHARED.lua_toboolean)(*self, arg) == 1
+	}
+
 	#[inline]
 	pub unsafe fn to_integer(&self, index: i32) -> LuaInt {
 		(LUA_SHARED.lua_tointeger)(*self, index)
@@ -375,7 +406,6 @@ impl LuaState {
 		None
 	}
 
-	#[cfg(debug_assertions)]
 	pub unsafe fn dump_stack(&self) {
 		let top = self.get_top();
 		println!("\n=== STACK DUMP ===");
@@ -390,13 +420,24 @@ impl LuaState {
 					self.pop();
 					str
 				}),
+				"boolean" => println!("{}. {}: {:?}", i, lua_type_name, {
+					self.push_value(i);
+					let bool = self.get_boolean(-1);
+					self.pop();
+					bool
+				}),
+				"number" => println!("{}. {}: {:?}", i, lua_type_name, {
+					self.push_value(i);
+					let n = self.to_number(-1);
+					self.pop();
+					n
+				}),
 				_ => println!("{}. {}", i, lua_type_name),
 			}
 		}
 		println!();
 	}
 
-	#[cfg(debug_assertions)]
 	pub unsafe fn dump_val(&self, index: i32) -> String {
 		let lua_type_name = self.lua_type_name(self.lua_type(index));
 		match lua_type_name.as_ref() {
